@@ -20,6 +20,11 @@ rae@cs.brynmawr.edu
 {-# OPTIONS_GHC -Wno-partial-type-signatures -Wno-redundant-constraints #-}
 #endif
 
+#if __GLASGOW_HASKELL__ >= 805
+{-# LANGUAGE DerivingVia #-}
+{-# LANGUAGE QuantifiedConstraints #-}
+#endif
+
 module Main where
 
 import Prelude hiding ( exp )
@@ -127,6 +132,10 @@ tests = test [ "sections" ~: $test1_sections  @=? $(dsSplice test1_sections)
         -- This one can't be tested by this means, because it contains an "undefined"
 #if __GLASGOW_HASKELL__ >= 803
              , "over_label" ~: $test46_overloaded_label @=? $(dsSplice test46_overloaded_label)
+#endif
+             , "do_partial_match" ~: $test47_do_partial_match @=? $(dsSplice test47_do_partial_match)
+#if __GLASGOW_HASKELL__ >= 805
+             , "quantified_constraints" ~: $test48_quantified_constraints @=? $(dsSplice test48_quantified_constraints)
 #endif
              ]
 
@@ -201,11 +210,13 @@ test_bug8884 = $(do info <- reify ''Poly
                     dinfo@(DTyConI (DOpenTypeFamilyD (DTypeFamilyHead _name _tvbs (DKindSig resK) _ann))
                                    (Just [DTySynInstD _name2 (DTySynEqn lhs _rhs)]))
                       <- dsInfo info
-                    case (resK, lhs) of
+                    let isTypeKind (DConT n) = isTypeKindName n
+                        isTypeKind _         = False
+                    case (isTypeKind resK, lhs) of
 #if __GLASGOW_HASKELL__ < 709
-                      (DStarT, [DVarT _]) -> [| True |]
+                      (True, [DVarT _]) -> [| True |]
 #else
-                      (DStarT, [DSigT (DVarT _) (DVarT _)]) -> [| True |]
+                      (True, [DSigT (DVarT _) (DVarT _)]) -> [| True |]
 #endif
                       _                                     -> do
                         runIO $ do
@@ -249,11 +260,58 @@ test_local_tyfam_expansion =
                    (expandType orig_ty)
        orig_ty `eqTHSplice` exp_ty)
 
+test_stuck_tyfam_expansion :: Bool
+test_stuck_tyfam_expansion =
+  $(do fam_name <- newName "F"
+       x        <- newName "x"
+       k        <- newName "k"
+       let orig_ty = DConT fam_name `DAppT` DConT '() -- F '()
+       exp_ty <- withLocalDeclarations
+                   (decsToTH [ -- type family F (x :: k) :: k
+                               DOpenTypeFamilyD
+                                 (DTypeFamilyHead fam_name
+                                                  [DKindedTV x (DVarT k)]
+                                                  (DKindSig (DVarT k))
+                                                  Nothing)
+                               -- type instance F (x :: ()) = x
+                             , DTySynInstD fam_name
+                                 (DTySynEqn [DSigT (DVarT x) (DConT ''())] (DVarT x))
+                             ])
+                   (expandType orig_ty)
+       orig_ty `eqTHSplice` exp_ty)
+
+test_t85 :: Bool
+test_t85 =
+  $(do let orig_ty =
+             (DConT ''Constant `DAppT` DConT ''Int `DAppT` DConT 'True)
+             `DSigT` (DConT ''Constant `DAppT` DConT ''Char `DAppT` DConT ''Bool)
+           expected_ty = DConT 'True `DSigT` DConT ''Bool
+       expanded_ty <- expandType orig_ty
+       expected_ty `eqTHSplice` expanded_ty)
+
+test_getDataD_kind_sig :: Bool
+test_getDataD_kind_sig =
+#if __GLASGOW_HASKELL__ >= 800
+  3 == $(do data_name <- newName "TestData"
+            a         <- newName "a"
+            let type_kind     = DConT typeKindName
+                data_kind_sig = DArrowT `DAppT` type_kind `DAppT`
+                                  (DArrowT `DAppT` type_kind `DAppT` type_kind)
+            (tvbs, _) <- withLocalDeclarations
+                           (decToTH (DDataD Data [] data_name [DPlainTV a]
+                                            (Just data_kind_sig) [] []))
+                           (getDataD "th-desugar: Impossible" data_name)
+            [| $(Syn.lift (length tvbs)) |])
+#else
+  True -- DataD didn't have the ability to store kind signatures prior to GHC 8.0
+#endif
+
 test_kind_substitution :: [Bool]
 test_kind_substitution =
   $(do a <- newName "a"
        b <- newName "b"
        c <- newName "c"
+       k <- newName "k"
        let subst = M.singleton a (DVarT b)
 
                  -- (Nothing :: Maybe a)
@@ -262,19 +320,28 @@ test_kind_substitution =
            ty2 = DForallT [DKindedTV c (DVarT a)] [] (DVarT c)
                  -- forall a (c :: a). c
            ty3 = DForallT [DPlainTV a, DKindedTV c (DVarT a)] [] (DVarT c)
+                 -- forall (a :: k) k (b :: k). Proxy b -> Proxy a
+           ty4 = DForallT [ DKindedTV a (DVarT k)
+                          , DPlainTV k
+                          , DKindedTV b (DVarT k)
+                          ] [] (DArrowT `DAppT` (DConT ''Proxy `DAppT` DVarT b)
+                                        `DAppT` (DConT ''Proxy `DAppT` DVarT a))
 
        substTy1 <- substTy subst ty1
        substTy2 <- substTy subst ty2
        substTy3 <- substTy subst ty3
+       substTy4 <- substTy subst ty4
 
        let freeVars1 = fvDType substTy1
            freeVars2 = fvDType substTy2
            freeVars3 = fvDType substTy3
+           freeVars4 = fvDType substTy4
 
            b1 = freeVars1 `eqTH` S.singleton b
            b2 = freeVars2 `eqTH` S.singleton b
            b3 = freeVars3 `eqTH` S.empty
-       [| [b1, b2, b3] |])
+           b4 = freeVars4 `eqTH` S.singleton k
+       [| [b1, b2, b3, b4] |])
 
 test_lookup_value_type_names :: [Bool]
 test_lookup_value_type_names =
@@ -395,6 +462,12 @@ main = hspec $ do
 
     it "doesn't expand local type families" $ test_local_tyfam_expansion
 
+    it "doesn't crash on a stuck type family application" $ test_stuck_tyfam_expansion
+
+    it "expands type synonyms in kinds" $ test_t85
+
+    it "reifies data type return kinds accurately" $ test_getDataD_kind_sig
+
     -- Remove map pprints here after switch to th-orphans
     zipWithM (\t t' -> it ("can do Type->DType->Type of " ++ t) $ t == t')
              $(sequence round_trip_types >>= Syn.lift . map pprint)
