@@ -26,6 +26,11 @@ rae@cs.brynmawr.edu
 {-# OPTIONS_GHC -Wno-orphans #-}  -- IsLabel is an orphan
 #endif
 
+#if __GLASGOW_HASKELL__ >= 805
+{-# LANGUAGE DerivingVia #-}
+{-# LANGUAGE QuantifiedConstraints #-}
+#endif
+
 {-# OPTIONS_GHC -fno-warn-missing-signatures -fno-warn-type-defaults
                 -fno-warn-name-shadowing #-}
 
@@ -105,31 +110,9 @@ eqTHSplice a b =
   then [| True |]
   else [| False |]
 
--- Note [Annotating list elements]
---
--- Type annotations on list elements are needed to satisfy GHC 8.0-rc1, otherwise
--- we get errors like:
---
---    Test/Run.hs:63:53: error:
---        • Couldn't match type ‘Maybe Integer’ with ‘forall a. Maybe a’
---          Expected type: [forall a. Maybe a]
---            Actual type: [Maybe Integer]
---        • In the second argument of ‘(:)’, namely
---            ‘(:) (Just 19) ((:) (Nothing :: Maybe Integer) [])’
---          In the second argument of ‘(:)’, namely
---            ‘(:) Nothing ((:) (Just 19) ((:) (Nothing :: Maybe Integer) []))’
---          In the second argument of ‘map’, namely
---            ‘(:)
---               (Just 1)
---               ((:) Nothing ((:) (Just 19) ((:) (Nothing :: Maybe Integer) [])))’
---
--- This is probably a bug in the GHC type checker, but I haven't been able to
--- reduce it yet
-
 test1_sections = [| map ((* 3) . (4 +) . (\x -> x * x)) [10, 11, 12] |]
 test2_lampats = [| (\(Just x) (Left z) -> x + z) (Just 5) (Left 10) |]
--- See Note [Annotating list elements]
-test3_lamcase = [| foldr (-) 0 (map (\case { Just x -> x ; Nothing -> (-3) }) [Just 1, Nothing :: Maybe Integer, Just 19, Nothing :: Maybe Integer]) |]
+test3_lamcase = [| foldr (-) 0 (map (\case { Just x -> x ; Nothing -> (-3) }) [Just 1, Nothing, Just 19, Nothing]) |]
 test4_tuples = [| (\(a, _) (# b, _ #) -> a + b) (1,2) (# 3, 4 #) |]
 test5_ifs = [| if (5 > 7) then "foo" else if | Nothing <- Just "bar", True -> "blargh" | otherwise -> "bum" |]
 test6_ifs2 = [| if | Nothing <- Nothing, False -> 3 | Just _ <- Just "foo" -> 5 |]
@@ -163,8 +146,7 @@ data InfixType = Int :+: Bool
   deriving (Show, Eq)
 
 test17_infixp = [| map (\(x :+: y) -> if y then x + 1 else x - 1) [5 :+: True, 10 :+: False] |]
--- See Note [Annotating list elements]
-test18_tildep = [| map (\ ~() -> Nothing :: Maybe Int) [undefined :: (), ()] |]
+test18_tildep = [| map (\ ~() -> Nothing :: Maybe Int) [undefined, ()] |]
 test19_bangp = [| map (\ !() -> 5) [()] |]
 test20_asp = [| map (\ a@(b :+: c) -> (if c then b + 1 else b - 1, a)) [5 :+: True, 10 :+: False] |]
 test21_wildp = [| zipWith (\_ _ -> 10) [1,2,3] ['a','b','c'] |]
@@ -173,19 +155,16 @@ test22_listp = [| map (\ [a,b,c] -> a + b + c) [[1,2,3],[4,5,6]] |]
 test23_sigp = [| map (\ (a :: Int) -> a + a) [5, 10] |]
 #endif
 
--- See Note [Annotating list elements]
-test24_fun = [| let f :: Maybe (Maybe a) -> Maybe a
-                    f (Just x) = x
+test24_fun = [| let f (Just x) = x
                     f Nothing = Nothing in
                 f (Just (Just 10)) |]
 
--- See Note [Annotating list elements]
 test25_fun2 = [| let f (Just x)
                        | x > 0 = x
                        | x < 0 = x + 10
                      f Nothing = 0
                      f _ = 18 in
-                 map f [Just (-5), Just 5, Just 10, Nothing :: Maybe Integer, Just 0] |]
+                 map f [Just (-5), Just 5, Just 10, Nothing, Just 0] |]
 
 test26_forall = [| let f :: Num a => a -> a
                        f x = x + 10 in
@@ -221,8 +200,8 @@ test35_expand = [| let f :: Pair a -> a
                        f = fst in
                    f |]
 
-type Const a b = b
-test36_expand = [| let f :: Const Int (,) Bool Char -> Char
+type Constant a b = b
+test36_expand = [| let f :: Constant Int (,) Bool Char -> Char
                        f = snd in
                    f |]
 
@@ -276,6 +255,15 @@ test46_overloaded_label = [| let p = Point 3 4 in
                              #x p - #y p |]
 #endif
 
+test47_do_partial_match = [| do { Just () <- [Nothing]; return () } |]
+
+#if __GLASGOW_HASKELL__ >= 805
+test48_quantified_constraints =
+  [| let f :: forall f a. (forall x. Eq x => Eq (f x), Eq a) => f a -> f a -> Bool
+         f = (==)
+     in f (Proxy @Int) (Proxy @Int) |]
+#endif
+
 type family TFExpand x
 type instance TFExpand Int = Bool
 type instance TFExpand (Maybe a) = [a]
@@ -338,14 +326,19 @@ dec_test_nums = [1..10] :: [Int]
 dec_test_nums = [1..11] :: [Int]
 #endif
 
-dectest1 = [d| data Dec1 = Foo | Bar Int |]
-dectest2 = [d| data Dec2 a = forall b. (Show b, Eq a) => MkDec2 a b Bool |]
-dectest3 = [d| data Dec3 a = forall b. MkDec3 { foo :: a, bar :: b }
+dectest1 = [d| data Dec1 where
+                 Foo :: Dec1
+                 Bar :: Int -> Dec1 |]
+dectest2 = [d| data Dec2 a where
+                 MkDec2 :: forall a b. (Show b, Eq a) => a -> b -> Bool -> Dec2 a |]
+dectest3 = [d| data Dec3 a where
+                 MkDec3 :: forall a b. { foo :: a, bar :: b } -> Dec3 a
 #if __GLASGOW_HASKELL__ >= 707
                type role Dec3 nominal
 #endif
                |]
-dectest4 = [d| newtype Dec4 a = MkDec4 (a, Int) |]
+dectest4 = [d| newtype Dec4 a where
+                 MkDec4 :: (a, Int) -> Dec4 a |]
 dectest5 = [d| type Dec5 a b = (a b, Maybe b) |]
 dectest6 = [d| class (Monad m1, Monad m2) => Dec6 (m1 :: * -> *) m2 | m1 -> m2  where
                  lift :: forall a. m1 a -> m2 a
@@ -353,16 +346,13 @@ dectest6 = [d| class (Monad m1, Monad m2) => Dec6 (m1 :: * -> *) m2 | m1 -> m2
 dectest7 = [d| type family Dec7 a (b :: *) (c :: Bool) :: * -> * |]
 dectest8 = [d| type family Dec8 a |]
 dectest9 = [d| data family Dec9 a (b :: * -> *) :: * -> * |]
-  -- NB: dectest9 inexplicably fails when you don't recompile everything from
-  -- scratch. I (Richard) haven't explored why. The failure is benign (replacing the
-  -- decl above with one with three variables) so I'm not terribly bothered.
 
 #if __GLASGOW_HASKELL__ < 707
 ds_dectest10 = DClosedTypeFamilyD
                  (DTypeFamilyHead
                     (mkName "Dec10")
                     [DPlainTV (mkName "a")]
-                    (DKindSig (DAppT (DAppT DArrowT DStarT) DStarT))
+                    (DKindSig (DAppT (DAppT DArrowT (DConT typeKindName)) (DConT typeKindName)))
                     Nothing)
                  [ DTySynEqn [DConT ''Int]  (DConT ''Maybe)
                  , DTySynEqn [DConT ''Bool] (DConT ''[]) ]
@@ -421,8 +411,19 @@ imp_inst_test1 = [d| instance Dec6 Maybe (Either ()) where
                        type M2 Maybe = Either () |]
 
 data family Dec9 a (b :: * -> *) :: * -> *
+#if __GLASGOW_HASKELL__ >= 800
+imp_inst_test2 = [d| data instance Dec9 Int Maybe a where
+                       MkIMB  ::             [a] -> Dec9 Int Maybe a
+                       MkIMB2 :: forall a b. b a -> Dec9 Int Maybe a |]
+imp_inst_test3 = [d| newtype instance Dec9 Bool m x where
+                       MkBMX :: m x -> Dec9 Bool m x |]
+#else
+-- TH-quoted data family instances with GADT syntax are horribly broken on GHC 7.10
+-- and older, so we opt to use non-GADT syntax on older GHCs so we can at least
+-- test *something*.
 imp_inst_test2 = [d| data instance Dec9 Int Maybe a = MkIMB [a] | forall b. MkIMB2 (b a) |]
 imp_inst_test3 = [d| newtype instance Dec9 Bool m x = MkBMX (m x) |]
+#endif
 
 type family Dec8 a
 imp_inst_test4 = [d| type instance Dec8 Int = Bool |]
@@ -511,6 +512,11 @@ reifyDecs = [d|
     r22 :: a -> a
     r22 = id   -- test #32
 
+  data R23 a = MkR23 { getR23 :: a }
+
+  r23Test :: R23 a -> a
+  r23Test (MkR23 { getR23 = x }) = x
+
 #if __GLASGOW_HASKELL__ >= 801
   pattern Point :: Int -> Int -> (Int, Int)
   pattern Point{x, y} = (x, y)
@@ -542,6 +548,14 @@ reifyDecs = [d|
   llEx :: [a] -> Int
   llEx LLMeth = 5
 #endif
+
+#if __GLASGOW_HASKELL__ >= 805
+  newtype Id a = MkId a
+    deriving stock Eq
+
+  newtype R24 a = MkR24 [a]
+    deriving Eq via (Id [a])
+#endif
   |]
 
 reifyDecsNames :: [Name]
